Scriptname idek:uss:USSUpdateScript extends ReferenceAlias

GlobalVariable Property _USSVersion Auto Const Mandatory
GlobalVariable Property _USSVersionCurrent Auto Const Mandatory
GlobalVariable Property _USSDebugMode Auto Const Mandatory
Quest Property _USSReset Auto Const Mandatory
Quest Property _USSQuest Auto Const Mandatory
Message Property _USSMessageUpdate127 Auto Const Mandatory
Message Property _USSMessageUpdateGeneral Auto Const Mandatory
Message Property _USSMessageUpdateUnknown Auto Const Mandatory

Event OnAliasReset()
	DoUpdate()
EndEvent

Event OnPlayerLoadGame()
	DoUpdate()
EndEvent

Function DoUpdate()
	if (_USSVersionCurrent.GetValue() != _USSVersion.GetValue())
		
		if (_USSDebugMode.getValue() > 1.9)
			Debug.MessageBox("USS running update script...\n\nLast version = " + _USSVersion.GetValue() + "\nCurrent version = " + _USSVersionCurrent.GetValue())
		endif
	
		if (_USSVersion.GetValue() == 0)
			; queue a reset if necessary (this also runs on new games)
			Update140()
			_USSMessageUpdateGeneral.Show()
		elseif (_USSVersion.GetValue() == 1.4)
			; no need to do this for updates from 1.3.x or below, because a full reset triggers in that case anyway
			Update142()
			_USSMessageUpdateGeneral.Show()
		elseif (_USSVersion.GetValue() < 1.5)
			; still 1.4.x, probably a minor update, so don't really do anything unless necessary
			Utility.Wait(5)
			_USSMessageUpdateGeneral.Show()
		else
			; version unrecognized - maybe downgrading?
			_USSMessageUpdateUnknown.Show()
		endif
		
		_USSVersion.SetValue(_USSVersionCurrent.GetValue())
	endif
EndFunction

Function Update140()
	; last version installed = 1.3.1 or below, OR new install
	; detect if aliases are correctly filled, if not do a reset
	bool doReset = true
	ReferenceAlias workshopAlias
	int i = 1
	int filledCount = 0
	
	Utility.Wait(5)

	while (i <= 100 && doReset)
		workshopAlias = _USSQuest.GetAlias(i) as ReferenceAlias
		if (workshopAlias.getReference() != None)
			filledCount += 1
		endif
		
		if filledCount > 25
			doReset = false
		endif

		i += 1
	endwhile

	if (doReset)
		_USSReset.Start()
		_USSMessageUpdate127.Show()
	endif
EndFunction

Function Update142()
	; added some extra pointers for speed in 1.4.2, so make sure those get filled or everything will be borked
	int i = 1
	idek:uss:USSScript USSScript
	ObjectReference workbench
	
	Utility.Wait(5)

	while (i <= 100)
		USSScript = _USSQuest.GetAlias(i) as idek:uss:USSScript
		workbench = USSScript.getReference()
		if (workbench != None)
			USSScript.FillPointers()
		endif
		
		i += 1
	endwhile
EndFunction
